'use strict';

exports.__esModule = true;
exports['default'] = paginate;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _lodashObjectMerge = require('lodash/object/merge');

var _lodashObjectMerge2 = _interopRequireDefault(_lodashObjectMerge);

var _lodashArrayUnion = require('lodash/array/union');

var _lodashArrayUnion2 = _interopRequireDefault(_lodashArrayUnion);

// Creates a reducer managing pagination, given the action types to handle,
// and a function telling how to extract the key from an action.

function paginate(_ref) {
  var types = _ref.types;
  var mapActionToKey = _ref.mapActionToKey;

  if (!Array.isArray(types) || types.length < 3) {
    throw new Error('Expected types to be an array of more than two elements.');
  }
  if (!types.every(function (t) {
    return typeof t === 'string';
  })) {
    throw new Error('Expected types to be strings.');
  }
  if (mapActionToKey && typeof mapActionToKey !== 'function') {
    throw new Error('Expected mapActionToKey to be a function.');
  }

  var requestType = types[0];
  var successType = types[1];
  var failureType = types[2];
  var createType = types[3];

  function updatePagination(state, action) {
    if (state === undefined) state = {
      isFetching: false,
      nextPageUrl: undefined,
      pageCount: 0,
      ids: []
    };

    var result = action.payload && action.payload.result;
    switch (action.type) {
      case requestType:
        return _lodashObjectMerge2['default']({}, state, {
          isFetching: true
        });
      case successType:
        return _lodashObjectMerge2['default']({}, state, {
          isFetching: false,
          ids: _lodashArrayUnion2['default'](state.ids, result),
          nextPageUrl: action.payload.nextPageUrl,
          pageCount: state.pageCount + 1
        });
      case failureType:
        return _lodashObjectMerge2['default']({}, state, {
          isFetching: false
        });
      case createType:
        return _lodashObjectMerge2['default']({}, state, {
          ids: _lodashArrayUnion2['default']([result], state.ids)
        });
      default:
        return state;
    }
  }

  return function updatePaginationByKey(state, action) {
    if (state === undefined) state = {};

    switch (action.type) {
      case requestType:
      case successType:
      case failureType:
      case createType:
        if (mapActionToKey) {
          var _merge;

          var key = mapActionToKey(action);
          if (typeof key !== 'string') {
            throw new Error('Expected key to be a string.');
          }
          return _lodashObjectMerge2['default']({}, state, (_merge = {}, _merge[key] = updatePagination(state[key], action), _merge));
        }
        return _lodashObjectMerge2['default']({}, state, updatePagination(state, action));
      default:
        return state;
    }
  };
}

module.exports = exports['default'];